@@ -42,7 +42,6 @@ private Q_SLOTS:
 void VirtualDesktopTest::initTestCase()
 {
     qRegisterMetaType<KWin::Window *>();
-    QSignalSpy applicationStartedSpy(kwinApp(), &Application::started);
     QVERIFY(waylandServer()->init(s_socketName));
     Test::setOutputConfig({
         QRect(0, 0, 1280, 1024),
@@ -54,7 +53,6 @@ void VirtualDesktopTest::initTestCase()
     qputenv("XKB_DEFAULT_RULES", "evdev");
 
     kwinApp()->start();
-    QVERIFY(applicationStartedSpy.wait());
 
 #if KWIN_BUILD_X11
     if (kwinApp()->x11Connection()) {
