@@ -31,6 +31,10 @@ QPointF CursorSource::hotspot() const
     return m_hotspot;
 }
 
+void CursorSource::frame(std::chrono::milliseconds timestamp)
+{
+}
+
 ShapeCursorSource::ShapeCursorSource(QObject *parent)
     : CursorSource(parent)
 {
@@ -61,12 +65,12 @@ void ShapeCursorSource::setShape(Qt::CursorShape shape)
     setShape(CursorShape(shape).name());
 }
 
-KXcursorTheme ShapeCursorSource::theme() const
+CursorTheme ShapeCursorSource::theme() const
 {
     return m_theme;
 }
 
-void ShapeCursorSource::setTheme(const KXcursorTheme &theme)
+void ShapeCursorSource::setTheme(const CursorTheme &theme)
 {
     if (m_theme != theme) {
         m_theme = theme;
@@ -105,7 +109,7 @@ void ShapeCursorSource::selectSprite(int index)
     if (m_currentSprite == index) {
         return;
     }
-    const KXcursorSprite &sprite = m_sprites[index];
+    const CursorSprite &sprite = m_sprites[index];
     m_currentSprite = index;
     m_image = sprite.data();
     m_size = QSizeF(m_image.size()) / m_image.devicePixelRatio();
@@ -126,6 +130,15 @@ SurfaceInterface *SurfaceCursorSource::surface() const
     return m_surface;
 }
 
+void SurfaceCursorSource::frame(std::chrono::milliseconds timestamp)
+{
+    if (m_surface) {
+        m_surface->traverseTree([&timestamp](SurfaceInterface *surface) {
+            surface->frameRendered(timestamp.count());
+        });
+    }
+}
+
 void SurfaceCursorSource::refresh()
 {
     m_size = m_surface->size();
