@@ -643,8 +643,8 @@ private:
 
     void updateOutputConfiguration();
     void updateOutputs(const std::optional<QList<Output *>> &outputOrder = std::nullopt);
-    void createDpmsFilter();
-    void maybeDestroyDpmsFilter();
+    void aboutToTurnOff();
+    void wakeUp();
     void assignBrightnessDevices();
 
     bool breaksShowingDesktop(Window *window) const;
